package golfCourse.agents;

import golfCourse.agents.contactToOTASystemAgent.handlers.GolfCourseAvailHandler;
import golfCourse.communication.ontology.GolfCourseOntology;
import jade.content.lang.Codec;
import jade.content.lang.sl.SLCodec;
import jade.content.onto.Ontology;
import jade.core.AID;
import jade.core.Agent;
import jade.domain.DFService;
import jade.domain.FIPAException;
import jade.domain.FIPANames;
import jade.util.Logger;

/**
 * All agents in Golf System extends this class.
 */
public abstract class GolfCourseAgent extends Agent{
	/**
	 * Ontology to exchange messages.
	 */
	public Ontology golfCourseOntology = GolfCourseOntology.getInstance();
	
	/**
	 * Codec used by agents to read and write messages.
	 */
	public Codec codec = new SLCodec();
	
	/**
	 * Language of messages in system.
	 */
	public String language = FIPANames.ContentLanguage.FIPA_SL0;
	
	/**
	 * AID of Manager Agent.
	 */
	public AID managerAID;
	
	/**
	 * Register language of messages in system.
	 */
	protected void registerLanguage()
	{
		getContentManager().registerLanguage(codec, language);
	}
	
	/**
	 * Register messages ontology.
	 */
	protected void registerOntology()
	{
		getContentManager().registerOntology(golfCourseOntology);
	}
	
	/**
	 * Constructor
	 * Registers language and ontology. 
	 */
	public GolfCourseAgent()
	{
		super();
		registerLanguage();
		registerOntology();
	}
	
	/**
	 * Register services in DF.
	 */
	protected abstract void registerInDF();
	
	/**
	 * Unregister services from DF.
	 */
	protected void unregisterInDF() throws FIPAException
	{
		 DFService.deregister(this); 
	}
	
	/**
	 * Get AID of Manager Agent.
	 */
	protected abstract void getManagerAID();
	
	protected abstract void takeDown();
}
