package golfCourse.agents;

import golfCourse.agents.managerAgent.behaviours.CustomerResponseBehaviour;
import golfCourse.communication.ontology.InfoMessage;
import golfCourse.communication.ontology.MA_generate_error_message;
import jade.content.onto.basic.Action;
import jade.core.AID;
import jade.core.behaviours.OneShotBehaviour;
import jade.lang.acl.ACLMessage;
import jade.util.Logger;

public class SendErrorToExternalBehaviour extends OneShotBehaviour{

	private AID managerAID;
	private String errorMessage;
	private String customerID;
	private GolfCourseAgent myAgent;
	
	private static Logger logger = Logger.getMyLogger(SendErrorToExternalBehaviour.class.getName());

	public SendErrorToExternalBehaviour(String errorMessage, GolfCourseAgent agent, String customerID)
	{
		this.managerAID = agent.managerAID;
		this.errorMessage = errorMessage;
		this.customerID = customerID;
		this.myAgent = agent;
	}
	
	public void action()
	{
		ACLMessage acl = new ACLMessage(ACLMessage.INFORM);
		
		MA_generate_error_message action = new MA_generate_error_message();
		
		InfoMessage infoMessage = new InfoMessage();
		
		infoMessage.setMessage(this.errorMessage);
		
		action.setError_msg(infoMessage);
		
		action.setCustomerMessageID(customerID);
		
		acl.setLanguage(myAgent.language);
		acl.setOntology(myAgent.golfCourseOntology.getName());
		acl.addReceiver(this.managerAID);
		
		try {
			myAgent.getContentManager().fillContent(acl, new Action(myAgent.getAID(), action));
			myAgent.send(acl);
		}
		catch(Exception e)
		{
			logger.info(myAgent.getLocalName() + ": cannot send OTA error message to MA " + e.getMessage());
		}
	}

}
