package golfCourse.agents;

import golfCourse.agents.managerAgent.behaviours.CustomerResponseBehaviour;
import golfCourse.communication.ontology.InfoMessage;
import golfCourse.communication.ontology.MA_show_error_message_action;
import jade.content.onto.basic.Action;
import jade.core.AID;
import jade.core.behaviours.OneShotBehaviour;
import jade.lang.acl.ACLMessage;
import jade.util.Logger;

public class SendInfoAboutErrorBehaviour extends OneShotBehaviour{
	
	private String message;
	private GolfCourseAgent myAgent;
	private AID managerAID;
	
	private static Logger logger = Logger.getMyLogger(SendInfoAboutErrorBehaviour.class.getName());

	
	public SendInfoAboutErrorBehaviour(String errorMessage, GolfCourseAgent agent)
	{
		this.managerAID = agent.managerAID;
		this.myAgent = agent;
		this.message = errorMessage;
	}
	
	public void action()
	{
		ACLMessage acl = new ACLMessage(ACLMessage.INFORM);
		
		MA_show_error_message_action action = new MA_show_error_message_action();
		
		InfoMessage infoMessage = new InfoMessage();
		infoMessage.setMessage(this.message);
		
		action.setMessage_(infoMessage);
		
		acl.setLanguage(myAgent.language);
		acl.setOntology(myAgent.golfCourseOntology.getName());
		acl.addReceiver(this.managerAID);
		
		try {
			myAgent.getContentManager().fillContent(acl, new Action(myAgent.getAID(), action));
			myAgent.send(acl);
		}
		catch(Exception e)
		{
			logger.info(myAgent.getLocalName() + ": cannot send info about error to MA - " + e.getMessage());
		}
	}
}
