package golfCourse.agents.contactToOTASystemAgent;

import golfCourse.agents.GolfCourseAgent;
import golfCourse.communication.ontology.GolfCourseOntology;
import jade.content.lang.sl.SLCodec;
import jade.core.AID;
import jade.domain.DFService;
import jade.domain.FIPAException;
import jade.domain.FIPANames;
import jade.domain.FIPAAgentManagement.DFAgentDescription;
import jade.domain.FIPAAgentManagement.ServiceDescription;
import jade.lang.acl.MessageTemplate;

import java.util.logging.Logger;


/**
 * Agent for contact with external OTA System.
 * This agent should
 * - send OTAGolfCourseAvailRQ message to another system,
 * - receive OTAGolfCourseAvailRS message from another system.
 * 
 * Now this agent only:
 * - receive OTAGolfCourseAvailRQ from Golf System,
 * - print this message on screen,
 * - send to Golf System OTAGolfCourseAvailRS (from example file).
 * 
 * This agent should be replaced by real agent for contact with external systems.
 * 
 */
public class ContactToOTASystemAgent extends GolfCourseAgent{
	
	private static Logger logger = Logger.getLogger(ContactToOTASystemAgent.class.getName());
	
	public final static String SERVICE_NAME = "Contact Test Agent Services";
	public final static String SERVICE_TYPE = "contact-service";
	public final static String SERVICE_OWNERSHIP = "Golf System";
	
	public ContactToOTASystemAgent()
	{
		super();
	}
	
	public void setup()
	{
		getContentManager().registerLanguage(new SLCodec(), FIPANames.ContentLanguage.FIPA_SL0);
		getContentManager().registerOntology(GolfCourseOntology.getInstance());

		// Initialize responding functionality.		
		MessageTemplate mt = MessageTemplate
				.MatchOntology(GolfCourseOntology.ONTOLOGY_NAME);
		
		addBehaviour(new ContactToOTASystemAgentResponder(this, mt));

		registerInDF();
		logger.info(getLocalName() + " is created");
	}
	
	protected void registerInDF()
	{
		try {
			ServiceDescription sd = getServiceDescription();			
			DFAgentDescription dfd = new DFAgentDescription();
			dfd.setName(getAID());
			dfd.addServices(sd);
			DFService.register(this, dfd);
			logger.info(getLocalName() + " is register in DF");
		} catch (FIPAException e) {
			logger.info(getLocalName() + "Cannot register agent at DF: " + e.getLocalizedMessage());
			doDelete();
		}
	}
	
	/**
	 * Returns information about the service this agent provides
	 * @return service description
	 */
	public static ServiceDescription getServiceDescription() {
		ServiceDescription sd = new ServiceDescription();
		sd.setType(SERVICE_TYPE);
		sd.setName(SERVICE_NAME);
		sd.setOwnership(SERVICE_OWNERSHIP);
		return sd;
	}
	
	protected void getManagerAID()
	{
		Object[] args = getArguments();
		if(args!= null)
		{
			if(args.length > 0)
			{
				this.managerAID = (AID)args[0];
				return;
			}
		}
	}
	
	protected void takeDown() 
    {
		try {
			unregisterInDF();
			logger.info(getLocalName() + " is unregistered");
		}
		catch(FIPAException e)
		{
			logger.info(getLocalName() + " cannot be unregistered: " + e.getLocalizedMessage());
		}
    }
}
