package golfCourse.agents.databaseAgent.handlers;

import golfCourse.agents.SendInfoAboutErrorBehaviour;
import golfCourse.agents.databaseAgent.DataBaseAgent;
import golfCourse.communication.ontology.DBA_save_golf_courses;
import golfCourse.communication.ontology.OWLData;
import golfCourse.communication.services.RequestHandler;
import golfCourse.dao.GolfCourseDAO;
import golfCourse.exceptions.DAOException;
import jade.content.onto.basic.Action;
import jade.core.Agent;
import jade.core.behaviours.DataStore;
import jade.domain.FIPAAgentManagement.FailureException;
import jade.lang.acl.ACLMessage;
import jade.util.Logger;

import com.hp.hpl.jena.ontology.OntModel;

/**
 * Load data behaviour.
 */
public class LoadDataHandler extends RequestHandler {
	
	private static Logger logger = Logger.getMyLogger(LoadDataHandler.class.getName());
	private DataBaseAgent myAgent;
	private OntModel mDomain;
	private OntModel mGolfCoursesDB;
	private GolfCourseDAO dao;
	
	public LoadDataHandler(Agent a, DataStore ds, Object requestKey,
			Object actKey, Object notificationKey) {
		super(a, ds, requestKey, actKey, notificationKey);
		
		myAgent = (DataBaseAgent)a;
	}

	protected final ACLMessage prepareResponse(ACLMessage request, Action act) throws FailureException {
		
		logger.info(myAgent.getLocalName() + " - load data action");
		this.mDomain = myAgent.getDomainModel();
		this.mGolfCoursesDB = myAgent.getGolfCoursesDBModel();
		this.dao = new GolfCourseDAO(mDomain, mGolfCoursesDB);
		
		// get ontologies from message from TA
		DBA_save_golf_courses action = (DBA_save_golf_courses)act.getAction();
		OWLData ontologies = action.getGolfCourses();
		
		try {
			dao.loadData(ontologies);
			logger.info(myAgent.getLocalName() + ": data loaded");
		} catch(DAOException e) {
			logger.info(myAgent.getLocalName() + ": error during loading data " + e.getLocalizedMessage());			
			myAgent.addBehaviour(new SendInfoAboutErrorBehaviour(myAgent.getLocalName() + ": error during loading data " + e.getMessage(), myAgent));
		}
		
		// no response
		return null;
	}
}
