package golfCourse.agents.managerAgent;

import golfCourse.agents.managerAgent.customer.behaviours.CloseSystemBehaviour;
import golfCourse.agents.managerAgent.customer.behaviours.CustomerRequestBehaviour;
import golfCourse.agents.managerAgent.customer.behaviours.LoadDataBehaviour;
import golfCourse.agents.managerAgent.customer.behaviours.RemoveDataBehaviour;
import golfCourse.properties.GolfCourseSystemProperties;
import jade.core.behaviours.CyclicBehaviour;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.MessageTemplate;
import jade.util.Logger;

import java.io.IOException;
import java.util.Properties;

public class ManagerAgentCustomerBehaviour extends CyclicBehaviour{
	
	private ManagerAgent myAgent;
	
	private final MessageTemplate customerMessage;
	
	private static Logger logger = Logger.getMyLogger(ManagerAgentCustomerBehaviour.class.getName());
	
	public ManagerAgentCustomerBehaviour(ManagerAgent myAgent)
	{
		this.myAgent = myAgent;
		
		customerMessage = 
		    MessageTemplate.MatchConversationId(myAgent.conversationID);
	}
	
	public void action() {		
		
		ACLMessage msg = myAgent.receive(customerMessage);
		
		if(msg != null)
		{
			String content = msg.getContent();
			
			if(content.equals(myAgent.loadDataMessage))
				myAgent.addBehaviour(new LoadDataBehaviour());
			else if(content.equals(myAgent.closeSystemMessage))
				myAgent.addBehaviour(new CloseSystemBehaviour(myAgent));
			else if(content.equals(myAgent.removeDataMessage))
				myAgent.addBehaviour(new RemoveDataBehaviour(myAgent));
			else
				myAgent.addBehaviour(new CustomerRequestBehaviour(msg, myAgent));
		}
	
		block();
	}
}
