package golfCourse.agents.managerAgent.behaviours;

import golfCourse.agents.contactToOTASystemAgent.ContactToOTASystemAgent;
import golfCourse.agents.databaseAgent.DataBaseAgent;
import golfCourse.agents.searchAgent.SearchAgent;
import golfCourse.agents.translationAgent.TranslationAgent;
import jade.core.behaviours.OneShotBehaviour;
import jade.util.Logger;
import jade.wrapper.AgentController;
import jade.wrapper.PlatformController;

/**
 * Start system - create all agent in system.
 */
public class StartSystemBehaviour extends OneShotBehaviour{
	
	private static Logger logger = Logger.getMyLogger(StartSystemBehaviour.class.getName());

	private int countTA = 1;
	private int countQA = 1;
	private int countCTA = 1;

    public StartSystemBehaviour(int countTA, int countQA)
    {
    	this.countTA = countTA;
    	this.countQA = countQA;
    }
	
	public void action()
	{
		logger.info(myAgent.getLocalName() + " - start system action");
		PlatformController container = myAgent.getContainerController(); 
	        
		Object[] args = new Object[1];
		args[0] = myAgent.getAID();
		
	    createTranslationAgents(container, args);   
	    createSearchAgents(container, args);
	    createDataBaseAgent(container, args);
	    createContactOTAAgent(container, args);
	}
	
	private void createTranslationAgents(PlatformController container, Object[] args)
	{
		try {
		 for(int i = 0; i < countTA; i++)
	        {
	        	AgentController controler = container.createNewAgent("TA"+"_"+i, TranslationAgent.class.getName(), args);
	        	controler.start();
	        	logger.info(myAgent.getLocalName() + " created and started new agent: TA"+i); 
	        }
		} catch(Exception e)
		{
			logger.info(myAgent.getLocalName() + ": error during creating translation agents " + e.getMessage());
		}
	}
	
	private void createContactOTAAgent(PlatformController container, Object[] args)
	{
		try {
		 for(int i = 0; i < countCTA; i++)
	        {
	        	AgentController controler = container.createNewAgent("CTA"+"_"+i, ContactToOTASystemAgent.class.getName(), args);
	        	controler.start();
	        	logger.info(myAgent.getLocalName() + " created and started new agent: CTA"+i); 
	        }
		} catch(Exception e)
		{
			logger.info(myAgent.getLocalName() + ": error during creating contact to OTA system agents " + e.getMessage());
		}
	}
	
	private void createSearchAgents(PlatformController container, Object[] args)
	{
		try {
			 for(int i = 0; i < countQA; i++)
		        {
		        	AgentController controler = container.createNewAgent("SA"+i, SearchAgent.class.getName(), args);
		        	controler.start();
		        	logger.info(myAgent.getLocalName() + " created and started new agent: SA"+i); 
		        }
			} catch(Exception e)
			{
				logger.info(myAgent.getLocalName() + ": error during creating search agents " + e.getMessage());
			}
	}
	
	private void createDataBaseAgent(PlatformController container, Object[] args)
	{
		try {
		     AgentController controler = container.createNewAgent("DBA",DataBaseAgent.class.getName(), args);
		     controler.start();
		     logger.info(myAgent.getLocalName() + " created and started new agent: DBA"); 
			} catch(Exception e)
			{
				logger.info(myAgent.getLocalName() + ": error during creating database agent " + e.getMessage());
			}
	}
	
	
	
}
