package golfCourse.agents.managerAgent.customer.behaviours;

import golfCourse.agents.managerAgent.ManagerAgent;
import golfCourse.agents.managerAgent.behaviours.ReceiveSystemErrorBehaviour;
import golfCourse.communication.ontology.OTAGolfCourseSearchRS_Message;
import golfCourse.communication.ontology.TA_translate_from_OTAGolfCourseSearchRS;
import golfCourse.properties.GolfCourseSystemProperties;
import jade.content.onto.basic.Action;
import jade.core.AID;
import jade.core.behaviours.OneShotBehaviour;
import jade.lang.acl.ACLMessage;
import jade.util.Logger;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;

/**
 * Load data from OTA xml file.
 * Read file and send it to Translation Agent.
 * TA will translate this file into instances of ontology and send its to DBA.
 * DBA will save these instances to database.
 */
public class LoadDataBehaviour extends OneShotBehaviour{

	String filePath = "";
	private static Logger logger = Logger.getMyLogger(LoadDataBehaviour.class.getName());
	
	public LoadDataBehaviour(String filePath)
	{
		this.filePath = filePath;
	}
	
	public LoadDataBehaviour()
	{
		this.filePath = GolfCourseSystemProperties.MANAGER_AGENT_GOLFCOURSE_SEARCH_RS;
	}
	
	public void action()
	{
		logger.info(myAgent.getLocalName() + " - load data action");
		try{
			String ota = readFile(filePath);
			AID aid = ((ManagerAgent)myAgent).translationService.getOne();
			
			if(aid == null)
			{
				myAgent.addBehaviour(new ReceiveSystemErrorBehaviour(myAgent.getLocalName() + ": no database agent in system", myAgent));
				return;
			}
			
			sendMessage(aid, ota);
		}catch(IOException e)
		{
			logger.info(myAgent.getLocalName() + " - exception during reading file with data " + e.getMessage());
		}
	}
	
	private void sendMessage(AID aid, String content) {
		
		 TA_translate_from_OTAGolfCourseSearchRS tfm = new TA_translate_from_OTAGolfCourseSearchRS();
		 OTAGolfCourseSearchRS_Message otaMsg = new OTAGolfCourseSearchRS_Message();
		 otaMsg.setOtaMessage(content);
		 tfm.setOtaGolfCourseSearchRS(otaMsg);
		 ACLMessage acl = new ACLMessage(ACLMessage.INFORM);
		 acl.setLanguage(((ManagerAgent)myAgent).language);
		 acl.setOntology(((ManagerAgent)myAgent).golfCourseOntology.getName());
		 
		 try {
			 myAgent.getContentManager().fillContent(acl, new Action(myAgent.getAID(), tfm));
			 acl.addReceiver(aid);
			 myAgent.send(acl);
			 logger.info(myAgent.getLocalName() + " sent OTA_GolfCourseRS to TA");
		 }
		 catch(Exception e)
		 {
			 logger.info(myAgent.getLocalName() + " - exception during sending OTAMessage to TA: " + e.getMessage());
		 }
	}
	
	public static String readFile(String filePath) throws IOException
	{
		String file = "";
        BufferedReader br= new BufferedReader(new InputStreamReader(new FileInputStream(filePath)));
        String line = br.readLine();
        while(line!=null) {
        	file = file + line;
        	line = br.readLine();
        }
        return file;
	}
}
