package golfCourse.agents.managerAgent.system.handlers;

import golfCourse.agents.managerAgent.ManagerAgent;
import golfCourse.agents.searchAgent.handlers.FindGolfCoursesHandler;
import golfCourse.communication.ontology.ErrorWarning;
import golfCourse.communication.ontology.GolfSystemOntologies;
import golfCourse.communication.ontology.MA_generate_error_message;
import golfCourse.communication.services.RequestHandler;
import jade.content.onto.basic.Action;
import jade.core.AID;
import jade.core.Agent;
import jade.core.behaviours.DataStore;
import jade.domain.FIPAAgentManagement.FailureException;
import jade.lang.acl.ACLMessage;
import jade.util.Logger;

public class GenerateErrorMessageHandler extends RequestHandler {

	private static Logger logger = Logger.getMyLogger(GenerateErrorMessageHandler.class.getName());
	
	protected ManagerAgent myAgent;
	
	public GenerateErrorMessageHandler(Agent a, DataStore ds, Object requestKey,
			Object actKey, Object notificationKey) {
		super(a, ds, requestKey, actKey, notificationKey);
		myAgent = (ManagerAgent) a;
	}

	protected final ACLMessage prepareResponse(ACLMessage request, Action act) throws FailureException {
		MA_generate_error_message action = (MA_generate_error_message)act.getAction();
		
		GolfSystemOntologies newAction = new GolfSystemOntologies();
		
		ErrorWarning error = new ErrorWarning();
		error.setContent(action.getError_msg().getMessage());
		error.setStatus("Unknown");
		newAction.addErrors(error);
		
		AID receiver = new AID(action.getCustomerMessageID(), AID.ISGUID);
		
		ACLMessage acl = new ACLMessage(ACLMessage.INFORM);
		acl.setLanguage(myAgent.language);
		acl.setOntology(myAgent.golfCourseOntology.getName());
		
		 try {
			 myAgent.getContentManager().fillContent(acl, new Action(myAgent.getAID(), newAction));
			 acl.addReceiver(receiver);
			 return acl;
		 }
		 catch(Exception e)
		 {
			 logger.info(myAgent.getLocalName() + " - exception during sending golf course avail map to TA: " + e.getMessage());
			 // TODO add system error behaviour
			 throw new FailureException(e.getMessage());
		 }
	}
}
