package golfCourse.agents.translationAgent.handlers;

import golfCourse.agents.SendErrorToOTABehaviour;
import golfCourse.agents.SendSystemErrorBehaviour;
import golfCourse.agents.translationAgent.TranslationAgent;
import golfCourse.communication.ontology.MA_send_OTAGolfCourseSearchRS;
import golfCourse.communication.ontology.OTAGolfCourseSearchRS_Message;
import golfCourse.communication.ontology.OWLData;
import golfCourse.communication.ontology.TA_translate_to_OTAGolfCourseSearchRS;
import golfCourse.communication.services.RequestHandler;
import golfCourse.exceptions.TranslateException;
import golfCourse.translations.toOTA.GolfCourseSearchRSTranslator;
import jade.content.lang.Codec;
import jade.content.onto.OntologyException;
import jade.content.onto.basic.Action;
import jade.core.AID;
import jade.core.Agent;
import jade.core.behaviours.DataStore;
import jade.domain.FIPAAgentManagement.FailureException;
import jade.lang.acl.ACLMessage;
import jade.util.Logger;

public class TranslateToOTASearchRSHandler extends RequestHandler {
	
	private static Logger logger = Logger.getMyLogger(TranslateToOTASearchRSHandler.class.getName());
	private TranslationAgent myAgent;
	
	public TranslateToOTASearchRSHandler(Agent a, DataStore ds, Object requestKey,
			Object actKey, Object notificationKey) {
		super(a, ds, requestKey, actKey, notificationKey);
		
		myAgent = (TranslationAgent)a;
	}

	protected final ACLMessage prepareResponse(ACLMessage request, Action act) throws FailureException {
		
		logger.info(myAgent.getLocalName() + " - translate to OTAGolfCourseSearchRS action");
		logger.info(myAgent.getLocalName() + " received ontologies from SA");

		TA_translate_to_OTAGolfCourseSearchRS action = (TA_translate_to_OTAGolfCourseSearchRS)act.getAction();
		
		OWLData queryResults = action.getGolfCourseOntologies();
		String messageID = action.getCustomerMessageID();
		
		GolfCourseSearchRSTranslator golfCourseSearch = new GolfCourseSearchRSTranslator();
		
		String ota = "";
		try {
			ota = golfCourseSearch.toGolfCourseSearchRS(queryResults);
		
			AID aid = ((TranslationAgent)myAgent).managerAID;
		
			ACLMessage aclMessage = new ACLMessage(ACLMessage.INFORM);
			MA_send_OTAGolfCourseSearchRS newAction = new MA_send_OTAGolfCourseSearchRS();
			newAction.setCustomerMessageID(messageID);
			OTAGolfCourseSearchRS_Message otaMessage = new OTAGolfCourseSearchRS_Message();
			otaMessage.setOtaMessage(ota);
			newAction.setMessage_OTAGolfCourseSearchRS(otaMessage);
			aclMessage.addReceiver(aid);
			
			aclMessage.setLanguage(((TranslationAgent)myAgent).language);
			aclMessage.setOntology(((TranslationAgent)myAgent).golfCourseOntology.getName());
	
			 myAgent.getContentManager().fillContent(aclMessage, new Action(myAgent.getAID(), newAction));
			 
			 logger.info(myAgent.getLocalName() + " sent OTAGolfCourseSearchRS message to MA");
			 return aclMessage;
		}
		catch(TranslateException e)
		{
			logger.info(myAgent.getLocalName() + ": exception during creating results from TA to MA: " + e.getMessage());
			myAgent.addBehaviour(new SendErrorToOTABehaviour(myAgent.getLocalName() + ": " + e.getMessage(), myAgent, action.getCustomerMessageID()));
		}
		catch(OntologyException e)
		{
			logger.info(myAgent.getLocalName() + ": " + e.getMessage());
			myAgent.addBehaviour(new SendSystemErrorBehaviour(myAgent.getLocalName() + ": " + e.getMessage(), myAgent));
		}
		catch(Codec.CodecException e)
		{
			logger.info(myAgent.getLocalName() + ": " + e.getMessage());
			myAgent.addBehaviour(new SendSystemErrorBehaviour(myAgent.getLocalName() + ": " + e.getMessage(), myAgent));
		}	
		return null;
	}
}
