package golfCourse.translations.toOTA;

import golfCourse.communication.ontology.OWLData;
import golfCourse.exceptions.TranslateException;
import golfCourse.ontologies.jastor.contacts.Contacts;
import golfCourse.ontologies.jastor.contacts.ContactsFactory;
import golfCourse.ontologies.jastor.golfCourse.GolfCourse;
import golfCourse.ontologies.jastor.golfCourse.GolfCourseFactory;
import golfCourse.ontologies.jastor.location.AddressRecord;
import golfCourse.ontologies.jastor.location.AddressRecordFactory;
import golfCourse.ontology.semantic.Memory;
import golfCourse.translations.ConditionsVocabulary;
import golfCourse.translations.castor.Address;
import golfCourse.translations.castor.CountryName;
import golfCourse.translations.castor.Errors;
import golfCourse.translations.castor.GolfCourses;
import golfCourse.translations.castor.OTA_GolfCourseSearchRS;
import golfCourse.translations.castor.OTA_GolfCourseSearchRSSequence;
import golfCourse.translations.castor.Phone;
import golfCourse.translations.castor.Success;
import golfCourse.translations.castor.Trait;
import golfCourse.translations.castor.Traits;

import java.io.InputStream;
import java.io.StringBufferInputStream;
import java.io.StringWriter;
import java.io.Writer;
import java.math.BigDecimal;
import java.util.Iterator;

import org.exolab.castor.xml.MarshalException;
import org.exolab.castor.xml.ValidationException;

import com.hp.hpl.jena.ontology.OntModel;
import com.hp.hpl.jena.rdf.model.Model;
import com.ibm.adtech.jastor.JastorException;

public class GolfCourseSearchRSTranslator {

	public GolfCourseSearchRSTranslator()
	{ }
	
	/**
	 * Translate results of search to OTA message.
	 * @param results
	 * @return OTA Message String
	 */
	public String toGolfCourseSearchRS(OWLData results) throws TranslateException
	{	
		OTA_GolfCourseSearchRS ota;
		
		if(results.getWithRootURI().size() == 0)
			ota = generateErrorGolfCourseRS();
		else 
			ota = generateValidGolfCourseRS(results);
		
		Writer writer = new StringWriter();
		
		try {
		ota.marshal(writer);
		} catch(MarshalException e)
		{
			throw new TranslateException("marshal exception" + e);
		}
		catch(ValidationException e2)
		{
			throw new TranslateException("validation exception" + e2);
		}
		
		return writer.toString();
	}
	
	/**
	 * Translate search results to OTA_GolfCourseSearchRS
	 * @param results
	 * @return OTA_GolfCourseSearchRS
	 */
	private OTA_GolfCourseSearchRS generateValidGolfCourseRS(OWLData results) throws TranslateException
	{
		OTA_GolfCourseSearchRS ota = new OTA_GolfCourseSearchRS();
		//
		
		//TODO change this
		ota.setVersion(BigDecimal.valueOf(322));
		
		OTA_GolfCourseSearchRSSequence sequence = new OTA_GolfCourseSearchRSSequence();
		
		GolfCourses golfCourses = getGolfCourses(results);
		sequence.setGolfCourses(golfCourses);
		
		Success success = new Success();
		sequence.setSuccess(success);
		
		ota.setOTA_GolfCourseSearchRSSequence(sequence);
		
		return ota;
	}
	
	/**
	 * Translate search results to castor.GolfCourses
	 * @param results
	 * @return GolfCourses
	 */
	private GolfCourses getGolfCourses(OWLData results) throws TranslateException
	{
		GolfCourses golfCourses = new GolfCourses();
		
		Memory memory = new Memory();
		memory.add(results);
		
		OntModel model = memory.getModel();
		
		for(Iterator iter = results.getAllWithRootURI(); iter.hasNext();)
		{
			try {
			GolfCourse golf = GolfCourseFactory.createGolfCourse((String)iter.next(), model);
			
			golfCourse.translations.castor.GolfCourse golfCourse = getGolfCourse(golf);
			
			String addressURI = golf.getAddress().uri();
			
			String contactURI = golf.getContactInfo().uri();
			
			AddressRecord addressRecord = AddressRecordFactory.createAddressRecord(addressURI, model);
			
			Address address = translateAddress(addressRecord);
			
			golfCourse.setAddress(address);
			
			Contacts contacts = ContactsFactory.createContacts(contactURI, model);
			
			Phone phone = translateContact(contacts);
			
			golfCourse.setPhone(phone);
			
			golfCourses.addGolfCourse(golfCourse);
			
			} catch(JastorException e)
			{
				throw new TranslateException("Translate to OTAGolfCourseSearchRS " + e.getLocalizedMessage());
			}
			
		}
		
		return golfCourses;
	}
	
	/**
	 * Translate search result to castor.GolfCourse
	 * @param model
	 * @param uri (URI of GolfCourse)
	 * @param modelString
	 * @return castor.GolfCourse
	 */
	private golfCourse.translations.castor.GolfCourse translateGolfCourse(Model model, String uri, String modelString) throws TranslateException
	{
		// = new golfCourse.translations.castor.GolfCourseSearchRS.GolfCourse();
		InputStream in = new StringBufferInputStream(modelString);
		model.read(in, uri);
		golfCourse.translations.castor.GolfCourse golfCourse;
	
		try {
			GolfCourse golf = GolfCourseFactory.createGolfCourse(uri, model);	
			golfCourse = getGolfCourse(golf);
			String addressURI = golf.getAddress().uri();
			golfCourse.setAddress(getAddress(modelString, addressURI, model));
			String contactURI = golf.getContactInfo().uri();
			golfCourse.setPhone(getPhone(modelString, contactURI, model));
		} catch(JastorException e)
		{
			throw new TranslateException("Jastor exception " + e.getLocalizedMessage());
		}
		
		return golfCourse;
	}
	
	private Phone getPhone(String modelString, String uri, Model model) throws JastorException
	{
		InputStream in = new StringBufferInputStream(modelString);
		model.read(in, uri);
		
		Contacts contacts = ContactsFactory.createContacts(uri, model);
		
		Phone phone = new Phone();
		phone.setPhoneNumber(contacts.getAddress());
		return phone;
	}
	
	private Phone translateContact(Contacts contacts) throws JastorException
	{
		Phone phone = new Phone();
		phone.setPhoneNumber(contacts.getAddress());
		return phone;
	}
	
	private Address getAddress(String modelString, String uri, Model model) throws JastorException
	{
		InputStream in = new StringBufferInputStream(modelString);
		model.read(in, uri);
		
		AddressRecord addressRecord = AddressRecordFactory.createAddressRecord(uri, model);

		Address address = new Address();
		
		address.setCityName(addressRecord.getCity());
		CountryName country = new CountryName();
		country.setContent(addressRecord.getCountry());
		address.setCountryName(country);
		
		return address;
	}
	
	private Address translateAddress(AddressRecord addressRecord) throws JastorException
	{
		Address address = new Address();
		
		address.setCityName(addressRecord.getCity());
		CountryName country = new CountryName();
		country.setContent(addressRecord.getCountry());
		address.setCountryName(country);
		
		return address;
	}
	
	private golfCourse.translations.castor.GolfCourse getGolfCourse(GolfCourse vGolfCourse) throws TranslateException
	{
		golfCourse.translations.castor.GolfCourse golfCourse = new golfCourse.translations.castor.GolfCourse();
		
		try {
			golfCourse.setName(vGolfCourse.getName());
			golfCourse.setID(vGolfCourse.getId());
			golfCourse.setTraits(translateTraits(vGolfCourse));
		} catch(JastorException e)
		{
			throw new TranslateException("Jastor exception " + e);
		}
		return golfCourse;
	}
	
	private Traits translateTraits(GolfCourse vGolfCourse) throws JastorException
	{
		Traits traits = new Traits();
		Trait trait;
		for(Iterator iter = vGolfCourse.getArchitect(); iter.hasNext();)
		{
			trait = new Trait();
			trait.setName(ConditionsVocabulary.OTA_ARCHITECT);
			trait.setValue((String)iter.next());
			traits.addTrait(trait);
		}
		
		String grassType = vGolfCourse.getGrassType();
		if(grassType != null)
		{
			trait = new Trait();
			trait.setName(ConditionsVocabulary.OTA_GRASS_TYPE);
			trait.setValue(grassType);
			traits.addTrait(trait);
		}
		
		Boolean availCaddy = vGolfCourse.getAvailCaddy();
		if(availCaddy != null)
		{
			trait = new Trait();
			trait.setName(ConditionsVocabulary.OTA_CADDIES_AVAILABLE);
			trait.setValue(availCaddy.toString());
			traits.addTrait(trait);
		}
		
	/*	trait = new Trait();
		trait.setName(GolfCourseVocabulary.OTA_ADA_CHALLENGED);
		trait.setValue(vGolfCourse.getADA);*/
		
		Boolean metalSpikes = vGolfCourse.getMetalSpikes();
		if(metalSpikes != null)
		{
			trait = new Trait();
			trait.setName(ConditionsVocabulary.OTA_METAL_SPIKES);
			trait.setValue(metalSpikes.toString());
			traits.addTrait(trait);
		}
		
		Boolean permCart = vGolfCourse.getPermCart();
		if(permCart != null)
		{
			trait = new Trait();
			trait.setName(ConditionsVocabulary.OTA_PERSONAL_CARTS);
			trait.setValue(permCart.toString());
			traits.addTrait(trait);
		}
		
		/*Object slope = vGolfCourse.getSlope();
		if(slope != null)
		{
			trait = new Trait();
			trait.setName(GolfCourseVocabulary.OTA_SLOPE);
			trait.setValue(slope.toString());
			traits.addTrait(trait);
		}*/
		
		Boolean singlesConfirmed = vGolfCourse.getSinglesConfirmed();
		if(singlesConfirmed != null)
		{
			trait = new Trait();
			trait.setName(ConditionsVocabulary.OTA_SINGLESS_CONFIRMED);
			trait.setValue(singlesConfirmed.toString());
			traits.addTrait(trait);
		}
		
		Float yardage = vGolfCourse.getYardage();
		if(yardage != null)
		{
			trait = new Trait();
			trait.setName(ConditionsVocabulary.OTA_YARDAGE);
			trait.setValue(yardage.toString());
			traits.addTrait(trait);
		}
		
		return traits;
	}
	
	/**
	 * Generate Error OTA_GolfCourseSearchRS
	 * @return OTA_GolfCourseSearchRS
	 */
	private OTA_GolfCourseSearchRS generateErrorGolfCourseRS()
	{
		OTA_GolfCourseSearchRS ota = new OTA_GolfCourseSearchRS();
		Errors errors = ErrorsRSTranslator.generateErrors("No golf course");
		ota.setErrors(errors);
		ota.setVersion(BigDecimal.valueOf(322));
		return ota;
	}
}
