package personalSystem.PersonalAgent;

import golfCourse.communication.ontology.GolfCourseOntology;
import golfCourse.communication.ontology.GolfSystemOntologies;
import golfCourse.communication.ontology.MA_GolfCourseAvailMap_receive;
import golfCourse.communication.ontology.MA_conditions_receive;
import golfCourse.communication.ontology.OWLData;
import jade.content.Concept;
import jade.content.ContentElement;
import jade.content.onto.basic.Action;
import jade.core.AID;
import jade.core.behaviours.CyclicBehaviour;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.MessageTemplate;
import jade.util.Logger;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.text.SimpleDateFormat;
import java.util.Date;

public class PersonalAgentBehaviour extends CyclicBehaviour{

		private PersonalAgent myAgent;
		
		private static Logger logger = Logger.getMyLogger(PersonalAgentBehaviour.class.getName());

		/**
		 * Message template from customer (user)
		 */
		private final MessageTemplate customerMessage; 
		       
		
		/**
		 * Message template from Manager Agent of Golf System
		 */
		private final MessageTemplate fromMAMessage;

		
		public PersonalAgentBehaviour(PersonalAgent myAgent)
		{
			this.myAgent = myAgent;
			customerMessage = MessageTemplate.MatchConversationId(((PersonalAgent)myAgent).customerConversationId);
			
			fromMAMessage = MessageTemplate.MatchOntology(GolfCourseOntology.getInstance().getName());
		}
		
		public void action()
		{
			block();
			
			ACLMessage msg = myAgent.receive(customerMessage);
			if(msg!=null)
			{
				if(msg.getContent().equals(((PersonalAgent)myAgent).generateMap))
				{
					logger.info(myAgent.getLocalName() + " received message from user.");
					sendMap();
				}
				else // generateConditions
				{	logger.info(myAgent.getLocalName() + "received message from user.");
					sendConditions();
					// send conditions
				}
			}
			
			msg = myAgent.receive(fromMAMessage);
			if(msg!=null)
			{
				try{
					ContentElement content = myAgent.getContentManager().extractContent(msg);
					Concept action = ((Action)content).getAction();
					//Concept action = (Action)content;
					
					if (action instanceof GolfSystemOntologies)
					{
						logger.info(myAgent.getLocalName() + " - message from MA");
						writeAnswer(((GolfSystemOntologies)action).getOwlData());
					}
					
				}catch(Exception e)
				{
					logger.info(myAgent.getLocalName() + " exception during reading message" + e.toString());
				}		
			}
		}
		
		/**
		 * Write answer to file results.xml
		 * @param ACLMessage - answer
		 */
		private void writeAnswer(OWLData ontologies) {
			
			String answer = ontologies.getDataModel();
			
			try {
				Date date = new Date();
				SimpleDateFormat formater = new SimpleDateFormat("yyyy-MM-dd_HH-mm-ss");
				
				FileWriter fileWriter = new FileWriter(PersonalAgentProperties.PERSONAL_SYSTEM_RESULTS_PATH + "/PA_results_"+formater.format(date) + ".txt");
				BufferedWriter bw = new BufferedWriter(fileWriter);
				bw.write(answer);
				bw.close();
				logger.info(myAgent.getLocalName() + " has written the answer.");
			} catch(Exception e) {
				logger.info(myAgent.getLocalName() + ": " + e.getLocalizedMessage());
			}
		}
		
		private void sendConditions() {
			golfCourse.communication.ontology.Conditions conditions;
			
			try {
				conditions = ConditionsGenerator.generateConditions();
			}
			catch(Exception e) {
				logger.info("Error during generating conditions: " + e.getMessage());
				return;
			}
			
			logger.info(myAgent.getLocalName() + " send message with conditions to MA");
			MA_conditions_receive action = new MA_conditions_receive();
			
			action.setConditions_(conditions);
			
			action.setCustomerMessageID(myAgent.getAID().getName());
			
			ACLMessage acl = new ACLMessage(ACLMessage.INFORM);
			acl.setLanguage(((PersonalAgent)myAgent).language);
			acl.setOntology(((PersonalAgent)myAgent).golfCourseOntology.getName());
			 
			 try {
				 myAgent.getContentManager().fillContent(acl, new Action(myAgent.getAID(), action));
				 acl.addReceiver(new AID("MA", AID.ISLOCALNAME));
				 myAgent.send(acl);
				 logger.info(myAgent.getLocalName() + " sent conditions to MA");
			 }
			 catch(Exception e)
			 {
				 logger.info("Exception during sending conditions to MA: " + e.getLocalizedMessage());
			 }
		}
		/**
		 * Send OTA message to Manager Agent
		 * @param ACLMessage to Manager Agent
		 */
		private void sendMap() {
			
			golfCourse.communication.ontology.Map map;
			
			try {
				map = MapGenerator.generate();
			}
			catch(Exception e) {
				logger.info("Error during generate map: " + e.getMessage());
				return;
			}
			
			logger.info(myAgent.getLocalName() + " send message to MA");
			MA_GolfCourseAvailMap_receive action = new MA_GolfCourseAvailMap_receive();
			
			action.setMap_(map);
			action.setCustomerMessageID(myAgent.getAID().getName());
			
			ACLMessage acl = new ACLMessage(ACLMessage.INFORM);
			acl.setLanguage(((PersonalAgent)myAgent).language);
			acl.setOntology(((PersonalAgent)myAgent).golfCourseOntology.getName());
			 
			 try {
				 myAgent.getContentManager().fillContent(acl, new Action(myAgent.getAID(), action));
				 acl.addReceiver(new AID("MA", AID.ISLOCALNAME));
				 myAgent.send(acl);
				 logger.info(myAgent.getLocalName() + " sent OTA_GolfCourseAvailMap to MA");
			 }
			 catch(Exception e)
			 {
				 logger.info("Exception during sending OTA_GolfCourseAvailMap to MA: " + e.getLocalizedMessage());
			 }
		}
}
